@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2015 Broad Institute, Inc.
+ * Copyright (c) 2012-2018 Broad Institute, Inc.
  *
  * This file is part of Pilon.
  *
@@ -14,6 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with Pilon.  If not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 package org.broadinstitute.pilon
@@ -45,6 +46,7 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
   var ambiguous = new Array[Boolean](size)
   var changed = new Array[Boolean](size)
   var deleted = new Array[Boolean](size)
+  var excluded = new Array[Boolean](size)
   //var lowCoverage = new Array[Boolean](size)
   //var lowConfidence = new Array[Boolean](size)
 
@@ -131,8 +133,17 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
 
   //val bams = Map.empty[Symbol, BamRegion]
   var bams = List[BamFile]()
+
   def bamsOfType(bamType: Symbol) = bams filter { _.bamType == bamType }
 
+  def nanoporeBams = bamsOfType('unpaired) filter { _.subType == 'nanopore}
+
+  def pacbioBams = bamsOfType('unpaired) filter { _.subType == 'pacbio }
+
+  def fragBams = bamsOfType('frags)
+
+  def longReadOnly = Pilon.longread && fragBams.isEmpty
+
   def initializePileUps = {
     pileUpRegion = new PileUpRegion(name, start, stop)
   }
@@ -203,6 +214,8 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
     val meanCoverage = pileUpRegion.coverage
     val nReads = pileUpRegion.readCount
 
+    if (longReadOnly) excludeMotifs()
+
     minDepth = {
       if (Pilon.minDepth >= 1) Pilon.minDepth.toInt
       else (Pilon.minDepth * meanCoverage).round.toInt max Pilon.minMinDepth
@@ -247,7 +260,7 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
             deleted(i + j) = true
             pileUpRegion(i + j).deletions += pu.deletions
           }
-        } else if (b != r) {
+        } else if (b != r && bc.score > 0) {
           // for ambiguous bases, fix them if --fix fixamb or if original base
           // not one of the top two alternatives
           if (homo) addChange(i, 'snp, pu)
@@ -304,38 +317,41 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
     var amb = 0
     for (i <- changeList) {
       val (kind, pu) = changeMap(i)
-      val rBase = refBase(locus(i))
+      val loc = locus(i)
+      val rBase = refBase(loc)
       val bc = pu.baseCall
       val cBase = bc.base
-      kind match {
-        case 'snp =>
-          if (fixSnps) snpFixList ::= (locus(i), rBase.toString, cBase.toString)
-          snps += 1
-        case 'amb =>
-          if (fixSnps) {
-            if (Pilon.iupac) {
-              // we put these on the small fix list because iupac codes can mess up assembly
-              // flank anchor kmers
-              smallFixList ::= (locus(i), rBase.toString, Bases.toIUPAC(cBase, bc.altBase).toString)
-            } else {
-              snpFixList ::= (locus(i), rBase.toString, cBase.toString)
+      if (!excluded(i)) {
+        kind match {
+          case 'snp =>
+            if (fixSnps) snpFixList ::= (loc, rBase.toString, cBase.toString)
+            snps += 1
+          case 'amb =>
+            if (fixSnps && !Pilon.longread) {
+              if (Pilon.iupac) {
+                // we put these on the small fix list because iupac codes can mess up assembly
+                // flank anchor kmers
+                smallFixList ::= (loc, rBase.toString, Bases.toIUPAC(cBase, bc.altBase).toString)
+              } else {
+                snpFixList ::= (loc, rBase.toString, cBase.toString)
+              }
+              amb += 1
             }
-          }
-          amb += 1
-        case 'ins =>
-          val insert = bc.insertion
-          if (fixIndels) smallFixList ::= (locus(i), "", insert)
-          ins += 1
-          insBases += insert.length
-        case 'del =>
-          val deletion = bc.deletion
-          if (fixIndels) smallFixList ::= (locus(i), deletion, "")
-          dels += 1
-          delBases += deletion.length
+          case 'ins =>
+            val insert = bc.insertion
+            if (fixIndels) smallFixList ::= (loc, "", insert)
+            ins += 1
+            insBases += insert.length
+          case 'del =>
+            val deletion = bc.deletion
+            if (fixIndels) smallFixList ::= (loc, deletion, "")
+            dels += 1
+            delBases += deletion.length
+        }
       }
       if (Pilon.verbose) logChange(i)
     }
-    
+
     // Report some stats
     val nConfirmed = confirmed count {x => x}
     val nonN = originalBases count {x => x != 'N'}
@@ -350,19 +366,19 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
     if (Pilon.fixList contains 'indels) log("; corrected ") else log("; found ")
     log(ins + " small insertions totaling " + insBases + " bases")
     logln(", " + dels + " small deletions totaling " + delBases + " bases")
-    
+
     // Report large collapsed regions (possible segmental duplication)
     val duplications = duplicationEvents
     if (duplications.size > 0) {
       for (d <- duplications) logln("Large collapsed region: " + d + " size " + d.size)
     }
-    
-    // Apply SNP fixes prior to reassemblies...it helps by giving better anchor sequence!  
+
+    // Apply SNP fixes prior to reassemblies...it helps by giving better anchor sequence!
     // We can't change coords here, though, so no indels!
     fixIssues(snpFixList)
 
     // Try to fill gaps
-    if ((Pilon.fixList contains 'gaps) && gaps.length > 0) {
+    if ((Pilon.fixList contains 'gaps) && gaps.nonEmpty) {
       logln("# Attempting to fill gaps")
       for (gap <- gaps) {
         val filler = new GapFiller(this)
@@ -376,7 +392,7 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
 
     // Try to reassemble around possible contiguity breaks, but stay away from gaps
     val breaks = possibleBreaks
-    if ((Pilon.fixList contains 'local) && !breaks.isEmpty) {
+    if ((Pilon.fixList contains 'local) && breaks.nonEmpty) {
       logln("# Attempting to fix local continuity breaks")
       for (break <- breaks) {
         val filler = new GapFiller(this)
@@ -395,6 +411,37 @@ class GenomeRegion(val contig: ReferenceSequence, start: Int, stop: Int)
     fixIssues(smallFixList ++ bigFixList)
   }
 
+  def homoRun(loc: Int): Int = {
+    val baseAtLoc = baseAt(loc)
+    for (i <- loc to stop) {
+      if (baseAt(i) != baseAtLoc) return i - loc
+    }
+    return 1 + stop - loc
+  }
+
+  def nanoporeExclude(loc: Int) = {
+    inRegion(loc-2) && inRegion(loc+2) &&
+      baseAt(loc - 2) == 'C' &&
+      baseAt(loc - 1) == 'C' &&
+      baseAt(loc + 1) == 'G' &&
+      baseAt(loc + 2) == 'G'
+  }
+
+  def excludeMotifs() = {
+    val pb = pacbioBams.nonEmpty
+    val nano = nanoporeBams.nonEmpty
+    val lr = pb || nano
+
+    for (i <- 0 until size)
+      excluded(i) = homoRun(locus(i)) >= 4 || (nano && nanoporeExclude(locus(i)))
+  }
+
+  def longReadChangeFilter(loc: Int): Boolean = {
+    if (homoRun(index(loc)) >= 4) false
+    else if (Pilon.nanopore && nanoporeExclude(index(loc))) false
+    else true
+  }
+
   val reassemblyFixes = Map.empty[Region, String]
 
   def closeCircle(estimatedLength: Int) = {
